# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/cgsense2023',
                'doc_host': 'https://hdocmsu.github.io',
                'git_url': 'https://github.com/hdocmsu/cgsense2023/',
                'lib_path': 'cgsense2023'},
  'syms': { 'cgsense2023.io': {'cgsense2023.io.read_data': ('P4_HPD_cg/io.html#read_data', 'cgsense2023/io.py')},
            'cgsense2023.math': { 'cgsense2023.math.crop_im': ('P4_HPD_cg/math.html#crop_im', 'cgsense2023/math.py'),
                                  'cgsense2023.math.im2kspace_fft2': ('P4_HPD_cg/math.html#im2kspace_fft2', 'cgsense2023/math.py'),
                                  'cgsense2023.math.kspace2im_ifft2': ('P4_HPD_cg/math.html#kspace2im_ifft2', 'cgsense2023/math.py'),
                                  'cgsense2023.math.kspace_filter': ('P4_HPD_cg/math.html#kspace_filter', 'cgsense2023/math.py'),
                                  'cgsense2023.math.radial_mask': ('P4_HPD_cg/math.html#radial_mask', 'cgsense2023/math.py'),
                                  'cgsense2023.math.rss_rec': ('P4_HPD_cg/math.html#rss_rec', 'cgsense2023/math.py'),
                                  'cgsense2023.math.zero_pad': ('P4_HPD_cg/math.html#zero_pad', 'cgsense2023/math.py')},
            'cgsense2023.metrics': { 'cgsense2023.metrics.mse': ('P4_HPD_cg/metrics.html#mse', 'cgsense2023/metrics.py'),
                                     'cgsense2023.metrics.nmse': ('P4_HPD_cg/metrics.html#nmse', 'cgsense2023/metrics.py'),
                                     'cgsense2023.metrics.nrmse': ('P4_HPD_cg/metrics.html#nrmse', 'cgsense2023/metrics.py'),
                                     'cgsense2023.metrics.print_metrics': ( 'P4_HPD_cg/metrics.html#print_metrics',
                                                                            'cgsense2023/metrics.py'),
                                     'cgsense2023.metrics.psnr': ('P4_HPD_cg/metrics.html#psnr', 'cgsense2023/metrics.py'),
                                     'cgsense2023.metrics.rmse': ('P4_HPD_cg/metrics.html#rmse', 'cgsense2023/metrics.py'),
                                     'cgsense2023.metrics.ssim': ('P4_HPD_cg/metrics.html#ssim', 'cgsense2023/metrics.py')},
            'cgsense2023.mri': { 'cgsense2023.mri.BaseOperator': ('P4_HPD_cg/mri.html#baseoperator', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.BaseOperator.__init__': ( 'P4_HPD_cg/mri.html#baseoperator.__init__',
                                                                            'cgsense2023/mri.py'),
                                 'cgsense2023.mri.BaseOperator.adjoint': ('P4_HPD_cg/mri.html#baseoperator.adjoint', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.BaseOperator.forward': ('P4_HPD_cg/mri.html#baseoperator.forward', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.MriImagingModel': ('P4_HPD_cg/mri.html#mriimagingmodel', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.MriImagingModel.__init__': ( 'P4_HPD_cg/mri.html#mriimagingmodel.__init__',
                                                                               'cgsense2023/mri.py'),
                                 'cgsense2023.mri.MriImagingModel.adjoint': ( 'P4_HPD_cg/mri.html#mriimagingmodel.adjoint',
                                                                              'cgsense2023/mri.py'),
                                 'cgsense2023.mri.MriImagingModel.forward': ( 'P4_HPD_cg/mri.html#mriimagingmodel.forward',
                                                                              'cgsense2023/mri.py'),
                                 'cgsense2023.mri.MriOperator': ('P4_HPD_cg/mri.html#mrioperator', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.MriOperator.__init__': ('P4_HPD_cg/mri.html#mrioperator.__init__', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.MriOperator.operator_lhs': ( 'P4_HPD_cg/mri.html#mrioperator.operator_lhs',
                                                                               'cgsense2023/mri.py'),
                                 'cgsense2023.mri.MriOperator.operator_rhs': ( 'P4_HPD_cg/mri.html#mrioperator.operator_rhs',
                                                                               'cgsense2023/mri.py'),
                                 'cgsense2023.mri.MriOperator.set_operator': ( 'P4_HPD_cg/mri.html#mrioperator.set_operator',
                                                                               'cgsense2023/mri.py'),
                                 'cgsense2023.mri.NuFFT': ('P4_HPD_cg/mri.html#nufft', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.NuFFT.__init__': ('P4_HPD_cg/mri.html#nufft.__init__', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.NuFFT._deapodizer_adj': ( 'P4_HPD_cg/mri.html#nufft._deapodizer_adj',
                                                                            'cgsense2023/mri.py'),
                                 'cgsense2023.mri.NuFFT._deapodizer_fwd': ( 'P4_HPD_cg/mri.html#nufft._deapodizer_fwd',
                                                                            'cgsense2023/mri.py'),
                                 'cgsense2023.mri.NuFFT._generate_grid_matrix': ( 'P4_HPD_cg/mri.html#nufft._generate_grid_matrix',
                                                                                  'cgsense2023/mri.py'),
                                 'cgsense2023.mri.NuFFT._grid_lut': ('P4_HPD_cg/mri.html#nufft._grid_lut', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.NuFFT._invgrid_lut': ('P4_HPD_cg/mri.html#nufft._invgrid_lut', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.NuFFT.adjoint': ('P4_HPD_cg/mri.html#nufft.adjoint', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.NuFFT.forward': ('P4_HPD_cg/mri.html#nufft.forward', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.calc_deapodizer': ('P4_HPD_cg/mri.html#calc_deapodizer', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.coil_sens': ('P4_HPD_cg/mri.html#coil_sens', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.coils_norm': ('P4_HPD_cg/mri.html#coils_norm', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.dcf': ('P4_HPD_cg/mri.html#dcf', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.hanning_window': ('P4_HPD_cg/mri.html#hanning_window', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.init_params': ('P4_HPD_cg/mri.html#init_params', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.kb_func': ('P4_HPD_cg/mri.html#kb_func', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.kb_kernel': ('P4_HPD_cg/mri.html#kb_kernel', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.prewhiten': ('P4_HPD_cg/mri.html#prewhiten', 'cgsense2023/mri.py'),
                                 'cgsense2023.mri.setup_params': ('P4_HPD_cg/mri.html#setup_params', 'cgsense2023/mri.py')},
            'cgsense2023.optimizer': { 'cgsense2023.optimizer.conjugate_gradient': ( 'P4_HPD_cg/optim.html#conjugate_gradient',
                                                                                     'cgsense2023/optimizer.py'),
                                       'cgsense2023.optimizer.frobenius_norm': ( 'P4_HPD_cg/optim.html#frobenius_norm',
                                                                                 'cgsense2023/optimizer.py'),
                                       'cgsense2023.optimizer.naive_steepest_descent': ( 'P4_HPD_cg/optim.html#naive_steepest_descent',
                                                                                         'cgsense2023/optimizer.py'),
                                       'cgsense2023.optimizer.steepest_descent': ( 'P4_HPD_cg/optim.html#steepest_descent',
                                                                                   'cgsense2023/optimizer.py')},
            'cgsense2023.plot': { 'cgsense2023.plot.gen_radial_traj': ('P4_HPD_cg/plot.html#gen_radial_traj', 'cgsense2023/plot.py'),
                                  'cgsense2023.plot.show_compared_images': ( 'P4_HPD_cg/plot.html#show_compared_images',
                                                                             'cgsense2023/plot.py'),
                                  'cgsense2023.plot.show_image_grid': ('P4_HPD_cg/plot.html#show_image_grid', 'cgsense2023/plot.py'),
                                  'cgsense2023.plot.show_trajectory': ('P4_HPD_cg/plot.html#show_trajectory', 'cgsense2023/plot.py')},
            'cgsense2023.utils': {}}}
